Attribute VB_Name = "MovePoly"
' Autor/Copyright: (c) K. Langbein for ActiveVB
'
' Purpose: This small example shows how you can move a polygon
' in a picturebox without the drag function. It demonstrates a
' simple way to find the position of an object by drawing
' objects on a hidden picture consequitvely and checking if the
' pixelcolor at the mouseposition has changed.
'
' The program is not supposed to be a complete CAD aplication!
' Feel free to complete it yourself.
  
Type pointapi

    X As Long
    Y As Long

End Type

Declare Function Polyline Lib "gdi32" (ByVal hdc As Long, _
                                       lpPoint As pointapi, _
                                       ByVal nCount As Long) As Long
                                       
Declare Function Polygon Lib "gdi32" (ByVal hdc As Long, _
                                      lpPoint As pointapi, _
                                      ByVal nCount As Long) As Long

Global Const pi = 3.14159265358979

Type Polygon_descriptor

    Xoff As Long         ' x offset
    Yoff As Long
    Xfact As Single      ' factor to act on data (zoom)
    Yfact As Single
    
    Forecolor As Long    ' colors
    Fillcolor As Long
    Drawwidth As Long
    
    Selected As Integer  ' flag to indicate object is selected
    Deleted As Integer   ' flag to indicate object is to be deleted
    Visible As Integer   ' visible/unvisible flag.
    Ptype As Integer     ' 2 = closed polygon, 1 = open polyline
    DataX() As Single    ' Array of polygon data
    DataY() As Single    ' to be redimensioned later.
    
End Type

' For VB newbies: Global means the same as Public
Global Poly() As Polygon_descriptor

Sub Create_API_Polygon(ByRef Plg As Polygon_descriptor, _
                       ByRef Apoly() As pointapi, _
                       ByVal dx As Long, _
                       ByVal dy As Long)
    
    ' Sub to convert real data to pixels as required by the
    ' API-GDI calls. The API functions Polygon and Polyline
    ' require a field of type pointapi. Apoly() is passed
    ' by reference thus the array is returned modified. dx and
    ' dy are used to draw with an extra offset during movement.
    
    Dim n As Long         ' number of points
    Dim i As Long         ' counter
    
    n = UBound(Plg.DataX)
    ReDim Apoly(n)        ' adapting size to number of points
                          ' of polygon
    
    For i = 1 To n
    
        Apoly(i).X = Plg.Xoff + Plg.Xfact * Plg.DataX(i) + dx
        Apoly(i).Y = Plg.Yoff - Plg.Yfact * Plg.DataY(i) + dy
        
    Next i
    
    ' I have the habit of leaving the first cell of any array
    ' empty so that I can use it as a joker. Thus Apoly(0) is
    ' not filled here.
    
End Sub

Sub draw_polygon(pic As Control, _
                 Plg As Polygon_descriptor, _
                 ByVal dx As Long, _
                 ByVal dy As Long)

    Dim ret As Long
    Dim n As Long
    Dim Apoly() As pointapi

    Call Create_API_Polygon(Plg, Apoly(), dx, dy)
    n = UBound(Apoly)

    Select Case Plg.Ptype
    
    Case 1
        
        ret = Polyline(pic.hdc, Apoly(1), n)
        
    Case 2
        ret = Polygon(pic.hdc, Apoly(1), n)
        
    Case 3
        ' who knows...
    End Select
    'pic.Refresh
    
End Sub
Sub create_sin(Pol As Polygon_descriptor)

    Dim nPi As Single    ' number of half periods
    Dim X As Single
    Dim Y As Single
    Dim a1 As Single     ' amplitude
    Dim a2 As Single     ' amplitude
    Dim a3 As Single     ' amplitude
    Dim Xstep As Single  ' step for increase of x
    Dim i As Long        ' counter
    Dim n As Long        ' number of points
    
    Xstep = 0.1
    nPi = 6
    n = 1 + nPi * pi / Xstep ' calculate the number of points required
    a1 = 1                   ' setting the amplitude
    
    ReDim Pol.DataX(n)
    ReDim Pol.DataY(n)
    
    For X = 0 To nPi * pi Step Xstep
    
        i = i + 1
        Pol.DataX(i) = X
        Pol.DataY(i) = a1 * Sin(X)
        
    Next X

End Sub

Sub create_ellipse(Pol As Polygon_descriptor)

    Dim nPi As Single    ' number of half periods
    Dim X As Single
    Dim Y As Single
    Dim a1 As Single     ' amplitude
    Dim a2 As Single     ' amplitude
    Dim a3 As Single     ' amplitude
    Dim Xstep As Single  ' step for increase of x
    Dim i As Long        ' counter
    Dim n As Long        ' number of points -1
    
    Xstep = 0.1
    nPi = 2
    n = (nPi * pi) / Xstep  ' calculate the number of points required
    a1 = 3                   ' setting the amplitude
    a2 = 1.5
    ReDim Pol.DataX(n + 1)
    ReDim Pol.DataY(n + 1)
    i = 0
    For X = 0 To nPi * pi Step Xstep
    
        i = i + 1
        Pol.DataX(i) = a1 * Sin(X)
        Pol.DataY(i) = a2 * Cos(X)
        
    Next X
    
    Pol.DataX(n + 1) = Pol.DataX(1)
    Pol.DataY(n + 1) = Pol.DataY(1)

End Sub

Sub create_circle(Pol As Polygon_descriptor)

    Dim nPi As Single    ' number of half periods
    Dim X As Single
    Dim Y As Single
    Dim a1 As Single     ' amplitude
    Dim a2 As Single     ' amplitude
    Dim a3 As Single     ' amplitude
    Dim Xstep As Single  ' step for increase of x
    Dim i As Long        ' counter
    Dim n As Long        ' number of points -1
    
    Xstep = 0.1
    nPi = 2
    n = (nPi * pi) / Xstep  ' calculate the number of points required
    a1 = 1                  ' setting the amplitude
    a2 = 1
    
    ReDim Pol.DataX(n + 1)
    ReDim Pol.DataY(n + 1)
    i = 0
    For X = 0 To nPi * pi Step Xstep
    
        i = i + 1
        Pol.DataX(i) = a1 * Sin(X)
        Pol.DataY(i) = a2 * Cos(X)
        
    Next X
    
    Pol.DataX(n + 1) = Pol.DataX(1)
    Pol.DataY(n + 1) = Pol.DataY(1)

End Sub


Sub create_flower(Pol As Polygon_descriptor)

    Dim nPi As Single    ' number of half periods
    Dim X As Single
    Dim Y As Single
    Dim a1 As Single     ' amplitude
    Dim a2 As Single     ' amplitude
    Dim a3 As Single     ' amplitude
    Dim Xstep As Single  ' step for increase of x
    Dim i As Long        ' counter
    Dim n As Long        ' number of points -1
    
    Xstep = 0.05
    nPi = 2
    n = (nPi * pi) / Xstep  ' calculate the number of points required
    a1 = 2.5                ' setting the amplitude
    a2 = a1 * 0.8           ' try out different ratios
    'a2 = a1 * 2            ' this is an interesting one
    
    ReDim Pol.DataX(n + 1)
    ReDim Pol.DataY(n + 1)
    
    i = 0
    For X = 0 To nPi * pi Step Xstep
    
        i = i + 1
        a3 = a2 * Sin(X * 5) + a1
        Pol.DataX(i) = a3 * Sin(X)
        Pol.DataY(i) = a3 * Cos(X)
        
    Next X
    
    Pol.DataX(n + 1) = Pol.DataX(1)
    Pol.DataY(n + 1) = Pol.DataY(1)

End Sub

Sub create_star(Pol As Polygon_descriptor)

    Dim nPi As Single    ' number of half periods
    Dim X As Single
    Dim Y As Single
    Dim a1 As Single     ' amplitude
    Dim a2 As Single     ' amplitude
    Dim a3 As Single     ' amplitude
    Dim Xstep As Single  ' step for increase of x
    Dim i As Long        ' counter
    Dim n As Long        ' number of points -1
    
    nPi = 2
    n = 12
    Xstep = 2 * pi / n ' calculate the number of points required
    a1 = 3             ' setting the amplitude
    a2 = 1
    
    ReDim Pol.DataX(n + 1)
    ReDim Pol.DataY(n + 1)
    
    i = 0
    For X = 0 To nPi * pi Step Xstep
    
        i = i + 1
        If i Mod 2 = 0 Then
            Pol.DataX(i) = a1 * Sin(X)
            Pol.DataY(i) = a1 * Cos(X)
        Else
            Pol.DataX(i) = a2 * Sin(X)
            Pol.DataY(i) = a2 * Cos(X)
        End If
        
    Next X
    
    Pol.DataX(n + 1) = Pol.DataX(1)
    Pol.DataY(n + 1) = Pol.DataY(1)
    
End Sub


Sub main()

    ReDim Poly(0) ' initilalizing the polygon array
    PlotF.Show    '
    TestF.Hide    ' The second form is not visible to the user
                  ' You can show it to check what is going on.
                  ' Of course you can use a second (hidden) picture
                  ' control on PlotF instead of placing it on an
                  ' additional Form
    
End Sub


